@@ -687,41 +687,84 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
     return(pars)
 }
 
-LoadGroupPars <- function(gmeans, gcov, estgmeans, estgcov, parnumber, parprior, Rasch = FALSE,
-                          customGroup = NULL){
-    if(!is.null(customGroup)){
-        par <- customGroup@par
-        parnum <- parnumber:(parnumber + length(par) - 1L)
-        customGroup@parnum <- parnum
-        return(customGroup)
+LoadGroupPars <- function(gmeans, gcov, estgmeans, estgcov, parnumber, parprior, dentype, Rasch = FALSE,
+                          customGroup = NULL, dcIRT_nphi = NULL, ...){
+    if (dentype != 'Davidian') {
+        if(!is.null(customGroup)){
+            par <- customGroup@par
+            parnum <- parnumber:(parnumber + length(par) - 1L)
+            customGroup@parnum <- parnum
+            return(customGroup)
+        } else {
+            nfact <- length(gmeans)
+            den <- function(obj, Theta){
+                gpars <- ExtractGroupPars(obj)
+                mu <- gpars$gmeans
+                sigma <- gpars$gcov
+                d <- mirt_dmvnorm(Theta, mean=mu, sigma=sigma)
+                d <- ifelse(d < 1e-300, 1e-300, d)
+                d
+            }
+            fn <- paste('COV_', 1L:nfact, sep='')
+            FNCOV <- outer(fn, 1L:nfact, FUN=paste, sep='')
+            FNMEANS <- paste('MEAN_', 1L:nfact, sep='')
+            tri <- lower.tri(gcov, diag=TRUE)
+            par <- c(gmeans, gcov[tri])
+            parnum <- parnumber:(parnumber + length(par) - 1L)
+            if(Rasch) diag(estgcov) <- TRUE
+            est <- c(estgmeans,estgcov[tri])
+            names(parnum) <- names(par) <- names(est) <- c(FNMEANS,FNCOV[tri])
+            tmp <- matrix(-Inf, nfact, nfact)
+            diag(tmp) <- 1e-4
+            lbound <- c(rep(-Inf, nfact), tmp[tri])
+            Nans <- rep(NaN,length(par))
+            if(dentype == 'mixture'){
+                par <- c(par, "PI" = 0)
+                est <- c(est, "PI" = TRUE)
+                parnum <- c(parnum, max(parnum)+1L)
+                Nans <- c(Nans, NaN)
+                lbound <- c(lbound, -Inf)
+            }
+            ubound <- rep(Inf, length(par))
+            ret <- new('GroupPars', par=par, est=est, parnames=names(est),
+                       nfact=nfact, any.prior=FALSE, den=den,
+                       safe_den=den, parnum=parnum, lbound=lbound, ubound=ubound,
+                       prior.type=rep(0L, length(par)), prior_1=Nans, prior_2=Nans, rrb=0, rrs=matrix(0),
+                       BFACTOR=FALSE, itemclass=0L, dentype=dentype)
+            return(ret)
+        }
     } else {
         nfact <- length(gmeans)
-        den <- function(obj, Theta){
+        if (nfact > 1) stop("Multidimensional DC-IRT models are not supported.")
+        # DC Density:
+        den <- function(obj, Theta) {
             gpars <- ExtractGroupPars(obj)
-            mu <- gpars$gmeans
-            sigma <- gpars$gcov
-            d <- mirt_dmvnorm(Theta, mean=mu, sigma=sigma)
-            d <- ifelse(d < 1e-300, 1e-300, d)
+            phi <- gpars$phi
+            d <- dcurver::ddc(Theta, phi = gpars$phi)
             d
         }
         fn <- paste('COV_', 1L:nfact, sep='')
         FNCOV <- outer(fn, 1L:nfact, FUN=paste, sep='')
         FNMEANS <- paste('MEAN_', 1L:nfact, sep='')
+        FNPHI <- paste('PHI_', 1L:dcIRT_nphi, sep='')
         tri <- lower.tri(gcov, diag=TRUE)
-        par <- c(gmeans, gcov[tri])
+        phi <- rep(1.570789, dcIRT_nphi)
+        estphi <- rep(T, dcIRT_nphi)
+        par <- c(gmeans, gcov[tri], phi)
         parnum <- parnumber:(parnumber + length(par) - 1L)
         if(Rasch) diag(estgcov) <- TRUE
-        est <- c(estgmeans,estgcov[tri])
-        names(parnum) <- names(par) <- names(est) <- c(FNMEANS,FNCOV[tri])
+        est <- c(estgmeans,estgcov[tri], estphi)
+        names(parnum) <- names(par) <- names(est) <- c(FNMEANS,FNCOV[tri],FNPHI)
         tmp <- matrix(-Inf, nfact, nfact)
         diag(tmp) <- 1e-4
-        lbound <- c(rep(-Inf, nfact), tmp[tri])
+        lbound <- c(rep(-Inf, nfact), tmp[tri], rep(-pi/2, dcIRT_nphi))
+        ubound <- c(rep(Inf, length(par)-dcIRT_nphi), rep(pi/2, dcIRT_nphi))
         Nans <- rep(NaN,length(par))
         ret <- new('GroupPars', par=par, est=est, parnames=names(est),
-                   nfact=nfact, any.prior=FALSE, den=den,
-                   safe_den=den, parnum=parnum, lbound=lbound, ubound=rep(Inf, length(par)),
-                   prior.type=rep(0L, length(par)), prior_1=Nans, prior_2=Nans, rrb=0, rrs=matrix(0),
-                   BFACTOR=FALSE, itemclass=0L)
+               nfact=nfact, any.prior=FALSE, den=den,
+               safe_den=den, parnum=parnum, lbound=lbound, ubound=ubound,
+               prior.type=rep(0L, length(par)), prior_1=Nans, prior_2=Nans, rrb=0, rrs=matrix(0),
+               BFACTOR=FALSE, itemclass=0L, dentype = 'Davidian')
         return(ret)
-    }
+      }
 }
