@@ -2,7 +2,7 @@ PrepData <- function(data, model, itemtype, guess, upper, gpcm_mats, opts,
                      parprior, verbose, technical, parnumber = 1, BFACTOR = FALSE,
                      grsm.block = NULL, rsm.block = NULL, mixed.design, customItems,
                      customGroup, fulldata = NULL, key, spline_args, internal_constraints,
-                     monopoly.k)
+                     monopoly.k, dentype, dcIRT_nphi)
 {
     if(is.null(grsm.block)) grsm.block <- rep(1, ncol(data))
     if(is.null(rsm.block)) rsm.block <- rep(1, ncol(data))
@@ -70,7 +70,7 @@ PrepData <- function(data, model, itemtype, guess, upper, gpcm_mats, opts,
         pick <- !sapply(gpcm_mats, is.null) & itemtype %in% c('gpcm', 'Rasch')
         tmp <- gpcm_mats[pick]
         if(!all(sapply(tmp, is.matrix)))
-            stop('Matricies must be used in gpcm_mats', call.=FALSE)
+            stop('Matrices must be used in gpcm_mats', call.=FALSE)
         if(!all(sapply(tmp, nrow) == K[pick])){
             nrows <- sapply(tmp, nrow)
             out <- !sapply(tmp, nrow) == K[pick]
@@ -104,7 +104,8 @@ PrepData <- function(data, model, itemtype, guess, upper, gpcm_mats, opts,
                            itemnames=itemnames, exploratory=exploratory, parprior=parprior,
                            parnumber=parnumber, BFACTOR=BFACTOR, mixed.design=mixed.design,
                            customItems=customItems, customGroup=customGroup, key=key,
-                           gpcm_mats=gpcm_mats, spline_args=spline_args, monopoly.k=monopoly.k)
+                           gpcm_mats=gpcm_mats, spline_args=spline_args, monopoly.k=monopoly.k,
+                           dcIRT_nphi=dcIRT_nphi, dentype=dentype)
     prodlist <- attr(pars, 'prodlist')
     exploratory <- attr(pars, 'exploratory')
     if(is(pars[[1L]], 'numeric') || is(pars[[1L]], 'logical')){
