@@ -12,6 +12,13 @@ thetaComb <- function(theta, nfact)
 	return(Theta)
 }
 
+thetaStack <- function(theta, nclass){
+    thetalist <- vector('list', nclass)
+    for(i in seq_len(nclass))
+        thetalist[[i]] <- theta
+    as.matrix(do.call(rbind, thetalist))
+}
+
 # Product terms
 prodterms <- function(theta0, prodlist)
 {
@@ -289,12 +296,33 @@ ExtractGroupPars <- function(x){
     if(x@itemclass < 0L) return(list(gmeans=0, gcov=matrix(1)))
     nfact <- x@nfact
     gmeans <- x@par[seq_len(nfact)]
-    tmp <- x@par[-seq_len(nfact)]
-    gcov <- matrix(0, nfact, nfact)
-    gcov[lower.tri(gcov, diag=TRUE)] <- tmp
-    if(nfact != 1L)
-        gcov <- gcov + t(gcov) - diag(diag(gcov))
-    return(list(gmeans=gmeans, gcov=gcov))
+    phi_matches <- grepl("PHI", x@parnames)
+    if (x@dentype == "Davidian") {
+        phi <- x@par[phi_matches]
+        tmp <- x@par[-c(seq_len(nfact), which(phi_matches))]
+        gcov <- matrix(0, nfact, nfact)
+        gcov[lower.tri(gcov, diag=TRUE)] <- tmp
+        if(nfact != 1L)
+            gcov <- gcov + t(gcov) - diag(diag(gcov))
+        return(list(gmeans=gmeans, gcov=gcov, phi=phi))
+    } else {
+        par <- x@par
+        if(x@dentype == "mixture") par <- par[-length(par)] # drop pi
+        tmp <- par[-seq_len(nfact)]
+        gcov <- matrix(0, nfact, nfact)
+        gcov[lower.tri(gcov, diag=TRUE)] <- tmp
+        if(nfact != 1L)
+            gcov <- gcov + t(gcov) - diag(diag(gcov))
+        return(list(gmeans=gmeans, gcov=gcov))
+    }
+}
+
+ExtractMixtures <- function(pars){
+    pick <- length(pars[[1L]])
+    logit_pi <- sapply(pars, function(x) x[[pick]]@par[length(x[[pick]]@par)])
+    max_logit_pi <- max(logit_pi)
+    pi <- exp(logit_pi - max_logit_pi)
+    pi / sum(pi)
 }
 
 reloadConstr <- function(par, constr, obj){
@@ -352,9 +380,7 @@ updatePrior <- function(pars, gTheta, list, ngroups, nfact, J,
                         dentype, sitems, cycles, rlist, lrPars = list(), full=FALSE,
                         MC = FALSE){
     prior <- Prior <- Priorbetween <- vector('list', ngroups)
-    if(dentype == 'EH'){
-        Prior[[1L]] <- list$EHPrior[[1L]]
-    } else if(dentype == 'custom'){
+    if(dentype == 'custom'){
         for(g in seq_len(ngroups)){
             gp <- pars[[g]][[J+1L]]
             Prior[[g]] <- gp@den(gp, gTheta[[g]])
@@ -371,6 +397,12 @@ updatePrior <- function(pars, gTheta, list, ngroups, nfact, J,
                 Prior[[g]] <- Prior[[g]] / sum(Prior[[g]])
             }
         }
+    } else if(dentype == 'Davidian'){
+        for(g in seq_len(ngroups)){
+            gp <- pars[[g]][[J+1L]]
+            Prior[[g]] <- gp@den(gp, gTheta[[g]])
+            Prior[[g]] <- Prior[[g]] / sum(Prior[[g]])
+        }
     } else {
         for(g in seq_len(ngroups)){
             gp <- ExtractGroupPars(pars[[g]][[J+1L]])
@@ -402,10 +434,16 @@ updatePrior <- function(pars, gTheta, list, ngroups, nfact, J,
             }
         }
     }
-    if(dentype == 'EH'){
+    if(dentype %in% c('EH', 'EHW')){
         if(cycles > 1L){
-            for(g in seq_len(ngroups))
-                Prior[[g]] <- rowSums(rlist[[g]][[1L]]) / sum(rlist[[g]][[1L]])
+            for(g in seq_len(ngroups)){
+                rr <- if(dentype == 'EHW') standardizeQuadrature(gTheta[[g]], pars[[g]][[J+1L]]@rr,
+                                                                 estmean=pars[[g]][[J+1L]]@est[1L],
+                                                                 estsd=pars[[g]][[J+1L]]@est[2L])
+                    else pars[[g]][[J+1L]]@rr
+                Prior[[g]] <- rr / sum(rr)
+                attr(Prior[[g]], 'mean_var') <- attr(rr, 'mean_var')
+            }
         } else {
             for(g in seq_len(ngroups)){
                 Prior[[g]] <- mirt_dmvnorm(gTheta[[g]], 0, matrix(1))
@@ -423,12 +461,16 @@ updatePrior <- function(pars, gTheta, list, ngroups, nfact, J,
             for(g in seq_len(ngroups))
                 Prior[[g]] <- matrix(rep(1 / length(gTheta[[g]])),
                                          nrow(lrPars@mus), nrow(gTheta[[g]]))
-
         } else {
             for(g in seq_len(ngroups))
                 Prior[[g]] <- matrix(rep(1 / length(Prior[[g]]), length(Prior[[g]])))
         }
     }
+    if(dentype == 'mixture'){
+        pis <- ExtractMixtures(pars)
+        for(g in seq_len(ngroups))
+            Prior[[g]] <- pis[g] * Prior[[g]]
+    }
     return(list(prior=prior, Prior=Prior, Priorbetween=Priorbetween))
 }
 
@@ -482,7 +524,7 @@ UpdateConstrain <- function(pars, constrain, invariance, nfact, nLambdas, J, ngr
                             for(j in seq_len(length(sel))){
                                 pick <- p[[sel[j]]]@parnum[names(p[[sel[j]]]@est) == picknames]
                                 if(!length(pick))
-                                    stop('CONSTRAIN = ... indexed a parameter that was not relavent for item ', sel[j],
+                                    stop('CONSTRAIN = ... indexed a parameter that was not relevant for item ', sel[j],
                                          call.=FALSE)
                                 constr <- c(constr, pick)
                             }
@@ -518,7 +560,7 @@ UpdateConstrain <- function(pars, constrain, invariance, nfact, nLambdas, J, ngr
                         for(g in groupsPicked){
                             pick <- pars[[g]][[j]]@parnum[names(pars[[g]][[j]]@est) == pickname]
                             if(!length(pick))
-                                stop('CONSTRAINB = ... indexed a parameter that was not relavent accross groups',
+                                stop('CONSTRAINB = ... indexed a parameter that was not relevant across groups',
                                      call.=FALSE)
                             constr <- c(constr, pick)
                         }
@@ -1275,11 +1317,11 @@ updateGrad <- function(g, L) L %*% g
 updateHess <- function(h, L) L %*% h %*% L
 
 makeopts <- function(method = 'MHRM', draws = 2000L, calcLL = TRUE, quadpts = NULL,
-                     SE = FALSE, verbose = TRUE, GenRandomPars,
+                     SE = FALSE, verbose = TRUE, GenRandomPars, dentype = 'Gaussian',
                      SEtol = .001, grsm.block = NULL, D = 1, TOL = NULL,
                      rsm.block = NULL, calcNull = FALSE, BFACTOR = FALSE,
                      technical = list(), hasCustomGroup = FALSE,
-                     SE.type = 'Oakes', large = NULL, accelerate = 'Ramsay', empiricalhist = FALSE,
+                     SE.type = 'Oakes', large = NULL, accelerate = 'Ramsay',
                      optimizer = NULL, solnp_args = list(), nloptr_args = list(), ...)
 {
     opts <- list()
@@ -1288,7 +1330,8 @@ makeopts <- function(method = 'MHRM', draws = 2000L, calcLL = TRUE, quadpts = NU
                 'gain', 'warn', 'message', 'customK', 'customPriorFun', 'customTheta', 'MHcand',
                 'parallel', 'NULL.MODEL', 'theta_lim', 'RANDSTART', 'MHDRAWS', 'removeEmptyRows',
                 'internal_constraints', 'SEM_window', 'delta', 'MHRM_SE_draws', 'Etable', 'infoAsVcov',
-                'PLCI', 'plausible.draws', 'storeEtable', 'keep_vcov_PD', 'Norder', 'MCEM_draws')
+                'PLCI', 'plausible.draws', 'storeEtable', 'keep_vcov_PD', 'Norder', 'MCEM_draws',
+                "zeroExtreme")
     if(!all(tnames %in% gnames))
         stop('The following inputs to technical are invalid: ',
              paste0(tnames[!(tnames %in% gnames)], ' '), call.=FALSE)
@@ -1305,6 +1348,21 @@ makeopts <- function(method = 'MHRM', draws = 2000L, calcLL = TRUE, quadpts = NU
                         'sandwich.Louis', 'Oakes', 'complete', 'SEM', 'Fisher', 'MHRM', 'FMHRM', 'numerical')))
         stop('SE.type argument not supported', call.=FALSE)
     if(!(method %in% c('EM', 'QMCEM', 'MCEM'))) accelerate <- 'none'
+    if(grepl('Davidian', dentype)){
+        tmp <- strsplit(dentype, '-')[[1]]
+        dentype <- tmp[1L]
+        opts$dcIRT_nphi <- as.integer(tmp[2L])
+        stopifnot(opts$dcIRT_nphi > 1L)
+    }
+    if(grepl('mixture', dentype)){
+        tmp <- strsplit(dentype, '-')[[1]]
+        dentype <- tmp[1L]
+        opts$ngroups <- as.integer(tmp[2L])
+        stopifnot(opts$n_mixture > 1L)
+    }
+    if(!(dentype %in% c('Gaussian', 'empiricalhist', 'discrete', 'empiricalhist_Woods', "Davidian",
+                        "EH", "EHW", 'mixture')))
+        stop('dentype not supported', call.=FALSE)
     opts$method = method
     if(draws < 1) stop('draws must be greater than 0', call.=FALSE)
     opts$draws = draws
@@ -1319,9 +1377,12 @@ makeopts <- function(method = 'MHRM', draws = 2000L, calcLL = TRUE, quadpts = NU
     opts$rsm.block = rsm.block
     opts$calcNull = calcNull
     opts$customPriorFun = technical$customPriorFun
-    opts$dentype <- 'Gaussian'
+    if(dentype == "empiricalhist") dentype <- 'EH'
+    if(dentype == "empiricalhist_Woods") dentype <- 'EHW'
+    opts$dentype <- dentype
+    opts$zeroExtreme <- FALSE
+    if(!is.null(technical$zeroExtreme)) opts$zeroExtreme <- technical$zeroExtreme
     if(BFACTOR) opts$dentype <- 'bfactor'
-    if(empiricalhist) opts$dentype <- 'EH'
     if(hasCustomGroup) opts$dentype <- 'custom'
     opts$accelerate = accelerate
     opts$Norder <- ifelse(is.null(technical$Norder), 2L, technical$Norder)
@@ -1362,13 +1423,24 @@ makeopts <- function(method = 'MHRM', draws = 2000L, calcLL = TRUE, quadpts = NU
     opts$internal_constraints  <- ifelse(is.null(technical$internal_constraints),
                                          TRUE, technical$internal_constraints)
     opts$keep_vcov_PD  <- ifelse(is.null(technical$keep_vcov_PD), TRUE, technical$keep_vcov_PD)
-    if(empiricalhist){
+    if(dentype == 'mixture'){
+        if(opts$method != 'EM')
+            stop('Mixture IRT densities only supported when method = \'EM\' ', call.=FALSE)
+        if(SE && !(SE.type %in% c('complete', 'Oakes')))
+            stop('Only Oakes and complete SE.types current supported for mixture models', call.=FALSE)
+    }
+    if(dentype %in% c("EH", 'EHW')){
         if(opts$method != 'EM')
             stop('empirical histogram method only applicable when method = \'EM\' ', call.=FALSE)
         if(opts$TOL == 1e-4) opts$TOL <- 3e-5
-        if(is.null(opts$quadpts)) opts$quadpts <- 199L
+        if(is.null(opts$quadpts)) opts$quadpts <- 121L
         if(opts$NCYCLES == 500L) opts$NCYCLES <- 2000L
     }
+    if(dentype == 'Davidian'){
+        if(opts$method != 'EM')
+            stop('Davidian curve method only applicable when method = \'EM\' ', call.=FALSE)
+        if(is.null(opts$quadpts)) opts$quadpts <- 121L
+    }
     if(is.null(opts$theta_lim)) opts$theta_lim <- c(-6,6)
     if(method == 'QMCEM' && is.null(opts$quadpts)) opts$quadpts <- 5000L
     if((opts$method %in% c('MHRM', 'MIXED') || SE.type == 'MHRM') && !GenRandomPars &&
@@ -1427,16 +1499,28 @@ makeopts <- function(method = 'MHRM', draws = 2000L, calcLL = TRUE, quadpts = NU
 }
 
 reloadPars <- function(longpars, pars, ngroups, J){
-    return(.Call('reloadPars', longpars, pars, ngroups, J,
-                      attr(pars[[1L]], 'nclasspars')))
+    .Call('reloadPars', longpars, pars, ngroups, J,
+          attr(pars[[1L]], 'nclasspars'))
 }
 
 computeItemtrace <- function(pars, Theta, itemloc, offterm = matrix(0L, 1L, length(itemloc)-1L),
-                             CUSTOM.IND){
-    itemtrace <- .Call('computeItemTrace', pars, Theta, itemloc, offterm)
-    if(length(CUSTOM.IND)){
-        for(i in CUSTOM.IND)
-            itemtrace[,itemloc[i]:(itemloc[i+1L] - 1L)] <- ProbTrace(pars[[i]], Theta=Theta)
+                             CUSTOM.IND, pis = NULL){
+    if(is.null(pis)){
+        itemtrace <- .Call('computeItemTrace', pars, Theta, itemloc, offterm)
+        if(length(CUSTOM.IND)){
+            for(i in CUSTOM.IND)
+                itemtrace[,itemloc[i]:(itemloc[i+1L] - 1L)] <- ProbTrace(pars[[i]], Theta=Theta)
+        }
+    } else {
+        tmp_itemtrace <- vector('list', length(pis))
+        for(g in seq_len(length(pis))){
+            tmp_itemtrace[[g]] <- .Call('computeItemTrace', pars[[g]]@ParObjects$pars, Theta, itemloc, offterm)
+            if(length(CUSTOM.IND)){
+                for(i in CUSTOM.IND)
+                    tmp_itemtrace[[g]][,itemloc[i]:(itemloc[i+1L] - 1L)] <- ProbTrace(pars[[g]]@ParObjects$pars[[i]], Theta=Theta)
+            }
+        }
+        itemtrace <- do.call(rbind, tmp_itemtrace)
     }
     return(itemtrace)
 }
@@ -1461,7 +1545,7 @@ loadESTIMATEinfo <- function(info, ESTIMATE, constrain, warn){
     acov <- try(solve(info), TRUE)
     if(is(acov, 'try-error')){
         if(warn)
-            warning('Could not invert information matrix; model likely is not identified.',
+            warning('Could not invert information matrix; model likely is not empirically identified.',
                     call.=FALSE)
         ESTIMATE$fail_invert_info <- TRUE
         return(ESTIMATE)
@@ -1500,7 +1584,7 @@ make.randomdesign <- function(random, longdata, covnames, itemdesign, N, LR=FALS
         if(any(grepl('\\*', splt[2L]) | grepl('\\+', splt[2L])))
             stop('The + and * operators are not supported. Please specify
                  which effects you want to interact with the : operator, and specify
-                 additional random effects in seperate list elements', call.=FALSE)
+                 additional random effects in separate list elements', call.=FALSE)
         gframe <- model.frame(as.formula(paste0('~',splt[2L])), longdata)
         sframe <- model.frame(as.formula(paste0('~',splt[1L])), longdata)
         levels <- interaction(gframe)
@@ -1570,7 +1654,7 @@ make.lrdesign <- function(df, formula, factorNames, EM=FALSE, TOL){
     inv_tXX <- try(solve(tXX), silent = TRUE)
     if(!is.nan(TOL)){
         if(is(inv_tXX, 'try-error'))
-            stop('Latent regression design matrix contains multicollinear terms.', call. = FALSE)
+            stop('Latent regression design matrix contains multi-collinear terms.', call. = FALSE)
     } else inv_tXX <- matrix(0, ncol(tXX), ncol(tXX))
     beta <- matrix(0, ncol(X), nfact)
     sigma <- matrix(0, nfact, nfact)
@@ -1647,6 +1731,21 @@ reloadRandom <- function(random, longpars){
     random
 }
 
+phi_bound <- function(phi, bound = c(-pi/2, pi/2)){
+    for(i in 1L:length(phi)){
+        while(TRUE){
+            if(phi[i] > bound[1L] && phi[i] <= bound[2L]) break
+            if(phi[i] < bound[1L]){
+                phi[i] <- phi[i] + pi
+            }
+            if(phi[i] >= bound[2L]){
+                phi[i] <- phi[i] - pi
+            }
+        }
+    }
+    phi
+}
+
 smooth.cor <- function(x){
     eig <- eigen(x)
     negvalues <- eig$values <= 0
@@ -1709,7 +1808,7 @@ BL.LL <- function(p, est, longpars, pars, ngroups, J, Theta, PrepList, specific,
         lrPars@beta[] <- lrPars@par
         lrPars@mus <- lrPars@X %*% lrPars@beta
     }
-    if(dentype == 'EH'){
+    if(dentype %in% c('EH', 'EHW')){
         Prior[[1L]] <- EHPrior[[1L]]
     } else if(dentype == 'custom'){
         for(g in seq_len(ngroups)){
@@ -2040,9 +2139,10 @@ loadSplinePars <- function(pars, Theta, MG = TRUE){
     }
     return(pars)
 }
-latentRegression_obj <- function(data, covdata, formula, empiricalhist, method){
+
+latentRegression_obj <- function(data, covdata, formula, dentype, method){
     if(!is.null(covdata) && !is.null(formula)){
-        if(empiricalhist)
+        if(dentype == "empiricalhist")
             stop('Empirical histogram method not supported with covariates', call.=FALSE)
         if(!is.data.frame(covdata))
             stop('covdata must be a data.frame object', call.=FALSE)
@@ -2101,6 +2201,27 @@ rmsea <- function(X2, df, N){
     ret
 }
 
+removeMissing <- function(obj){
+    dat <- extract.mirt(obj, 'data')
+    obj@Data$data <- na.omit(dat)
+    pick <- attr(obj@Data$data, 'na.action')
+    obj@Data$group <- obj@Data$group[-pick]
+    ind1 <- 0L
+    for(g in seq_len(length(obj@Data$groupNames))){
+        ind2 <- 1L:nrow(obj@Data$fulldata[[g]]) + ind1
+        pick2 <- ind2 %in% pick
+        ind1 <- nrow(obj@Data$fulldata[[g]]) + ind1
+        obj@Data$fulldata[[g]] <- obj@Data$fulldata[[g]][!pick2, , drop=FALSE]
+    }
+    if(is(obj, 'MultipleGroupClass')){
+        for(g in seq_len(length(obj@Data$groupNames))){
+            obj@ParObjects$pars[[g]]@Data$data <- dat[obj@Data$groupNames[g] == obj@Data$group,
+                                                         , drop=FALSE]
+        }
+    }
+    obj
+}
+
 controlCandVar <- function(PA, cand, min = .1, max = .6){
     if(PA > max) cand <- cand * 1.05
     else if(PA < min) cand <- cand * 0.9
