@@ -234,6 +234,120 @@ RcppExport SEXP buildXi2els(SEXP Rdim1, SEXP Rdim2, SEXP Rnitems, SEXP REIs,
     END_RCPP
 }
 
+RcppExport SEXP buildXi2els_C2(SEXP Rdim1, SEXP Rdim2, SEXP Rnitems0, 
+	SEXP Rnitems, SEXP RPIs, SEXP REIs, SEXP REIs2, SEXP RPrior)
+{
+    BEGIN_RCPP
+    const int dim1 = as<int>(Rdim1);
+    const int dim2 = as<int>(Rdim2);
+    const int nitems0 = as<int>(Rnitems0);
+    const int nitems = as<int>(Rnitems);
+    const NumericMatrix PIs(RPIs);
+    const NumericMatrix EIs(REIs);
+    const NumericMatrix EIs2(REIs2);
+    const vector<double> Prior = as< vector<double> >(RPrior);
+    const int N = EIs.nrow();
+    NumericMatrix Xi11(dim1, dim1);
+    NumericMatrix Xi12(dim1, dim2);
+    NumericMatrix Xi22(dim2, dim2);
+
+    for(int i = 0; i < nitems0; ++i){
+        for(int j = 0; j < nitems0; ++j){
+            if(i >= j){
+                double pa = 0, pb = 0, pab = 0;
+                for(int n = 0; n < N; ++n){
+                    pa += PIs(n,i) * Prior[n];
+                    pb += PIs(n,j) * Prior[n];
+                }
+                if(i == j){
+                    for(int n = 0; n < N; ++n)
+                        pab += PIs(n,i) * Prior[n];
+                } else {
+                    for(int n = 0; n < N; ++n)
+                		pab += PIs(n,i) * PIs(n,j) * Prior[n];
+                }
+                Xi11(i,j) = pab - pa*pb;
+                Xi11(j,i) = Xi11(i,j);
+            }
+        }
+    }
+    for(int k = 0; k < nitems0; ++k){
+    	int ind = 0;
+        for(int i = 0; i < nitems; ++i){
+            for(int j = 0; j < nitems; ++j){
+                if(i < j){
+                    double pab = 0, pc = 0, pabc = 0;
+                    for(int n = 0; n < N; ++n){
+                        pab += EIs(n,i) * EIs(n,j) * Prior[n];
+                        pc += PIs(n,k) * Prior[n];
+                    } 
+                    if(i == k){
+                        for(int n = 0; n < N; ++n)
+                            pabc += EIs2(n,i) * EIs(n,j) * Prior[n];
+                    } else if(j == k){
+                        for(int n = 0; n < N; ++n)
+                            pabc += EIs(n,i) * EIs2(n,j) * Prior[n];
+                    } else {
+                        for(int n = 0; n < N; ++n)
+                    		pabc += EIs(n,i) * EIs(n,j) * PIs(n,k) * Prior[n];
+                    }
+                    Xi12(k,ind) = pabc - pab*pc;
+                    ++ind;
+                }
+            }
+        }
+    }
+    int ind1 = 0;
+    for(int k = 0; k < nitems; ++k){
+        for(int l = 0; l < nitems; ++l){
+            if(k < l){
+                int ind2 = 0;
+                for(int i = 0; i < nitems; ++i){
+                    for(int j = 0; j < nitems; ++j){
+                        if(i < j){
+                            double pab = 0, pcd = 0, pabcd = 0;
+                            for(int n = 0; n < N; ++n){
+                                pab += EIs(n,i) * EIs(n,j) * Prior[n];
+                                pcd += EIs(n,k) * EIs(n,l) * Prior[n];
+                            }
+                            if((i == k) & (j == l)){
+                                for(int n = 0; n < N; ++n)
+                                    pabcd += EIs2(n,i) * EIs2(n,j) * Prior[n];
+                            } else if(i == k){
+                                for(int n = 0; n < N; ++n)
+                                    pabcd += EIs2(n,i) * EIs(n,j) * EIs(n,l) * Prior[n];
+                            } else if(j == k){
+                                for(int n = 0; n < N; ++n)
+                                    pabcd += EIs(n,i) * EIs2(n,j) * EIs(n,l) * Prior[n];
+                            } else if(i == l){
+                                for(int n = 0; n < N; ++n)
+                                    pabcd += EIs2(n,i) * EIs(n,j) * EIs(n,k) * Prior[n];
+                            } else if(j == l){
+                                for(int n = 0; n < N; ++n)
+                                    pabcd += EIs(n,i) * EIs2(n,j) * EIs(n,k) * Prior[n];
+                            } else {
+                                for(int n = 0; n < N; ++n)
+                                    pabcd += EIs(n,i) * EIs(n,j) * EIs(n,k) * EIs(n,l) * Prior[n];
+                            }
+                            Xi22(ind1, ind2) = pabcd - pab*pcd;
+                            ++ind2;
+                        }
+                    }
+                }
+                ++ind1;
+            }
+        }
+    }
+
+    List ret;
+    ret["Xi11"] = Xi11;
+    ret["Xi12"] = Xi12;
+    ret["Xi22"] = Xi22;
+    return(ret);
+
+    END_RCPP
+}
+
 double antilogit(const double *x){
     double ret;
     if(*x > 998.0) ret = 1.0;
